@@ -238,9 +238,14 @@ struct predicate
   /* The precedence of this node.  Only has meaning for operators. */
   enum predicate_precedence p_prec;
 
-  /* True if this predicate node produces side effects. */
+  /* True if this predicate node produces side effects.
+     If side_effects are produced
+     then optimization will not be performed */
   boolean side_effects;
 
+  /* True if this predicate node requires default print be turned off. */
+  boolean no_default_print;
+
   /* True if this predicate node requires a stat system call to execute. */
   boolean need_stat;
 
@@ -403,10 +408,10 @@ extern boolean stay_on_filesystem;
 extern boolean stop_at_current_level;
 extern boolean have_stat;
 extern char *rel_pathname;
-#ifndef HAVE_FCHDIR
-extern char *starting_dir;
-#else
+extern char const *starting_dir;
 extern int starting_desc;
+#if ! defined HAVE_FCHDIR && ! defined fchdir
+# define fchdir(fd) (-1)
 #endif
 extern int exit_status;
 extern int path_length;
