@@ -58,6 +58,7 @@ static void process_top_path PARAMS((char *pathname));
 static int process_path PARAMS((char *pathname, char *name, boolean leaf, char *parent));
 static void process_dir PARAMS((char *pathname, char *name, int pathlen, struct stat *statp, char *parent));
 static boolean no_side_effects PARAMS((struct predicate *pred));
+static boolean default_prints PARAMS((struct predicate *pred));
 
 /* Name this program was run with. */
 char *program_name;
@@ -107,21 +108,23 @@ boolean stay_on_filesystem;
    Can be set by -prune, -maxdepth, and -xdev/-mount. */
 boolean stop_at_current_level;
 
-#ifndef HAVE_FCHDIR
-/* The full path of the initial working directory.  */
-char *starting_dir;
-#else
+/* The full path of the initial working directory, or "." if
+   STARTING_DESC is nonnegative.  */
+char const *starting_dir = ".";
+
 /* A file descriptor open to the initial working directory.
    Doing it this way allows us to work when the i.w.d. has
    unreadable parents.  */
 int starting_desc;
-#endif
+
+/* The stat buffer of the initial working directory. */
+struct stat starting_stat_buf;
 
 /* If true, we have called stat on the current path. */
 boolean have_stat;
 
 /* The file being operated on, relative to the current directory.
-   Used for stat, readlink, and opendir.  */
+   Used for stat, readlink, remove, and opendir.  */
 char *rel_pathname;
 
 /* Length of current path. */
@@ -151,7 +154,7 @@ int
 main (int argc, char **argv)
 {
   int i;
-  PFB parse_function;		/* Pointer to who is to do the parsing. */
+  PFB parse_function;		/* Pointer to the function which parses. */
   struct predicate *cur_pred;
   char *predicate_name;		/* Name of predicate being parsed. */
 
@@ -223,7 +226,7 @@ main (int argc, char **argv)
       free ((char *) cur_pred);
       parse_print (argv, &argc);
     }
-  else if (!no_side_effects (predicates->pred_next))
+  else if (!default_prints (predicates->pred_next))
     {
       /* One or more predicates that produce output were given;
 	 remove the unneeded initial `('. */
@@ -262,15 +265,20 @@ main (int argc, char **argv)
   print_tree (eval_tree, 0);
 #endif /* DEBUG */
 
-#ifndef HAVE_FCHDIR
-  starting_dir = xgetcwd ();
-  if (starting_dir == NULL)
-    error (1, errno, _("cannot get current directory"));
-#else
   starting_desc = open (".", O_RDONLY);
+  if (0 <= starting_desc && fchdir (starting_desc) != 0)
+    {
+      close (starting_desc);
+      starting_desc = -1;
+    }
   if (starting_desc < 0)
-    error (1, errno, _("cannot open current directory"));
-#endif
+    {
+      starting_dir = xgetcwd ();
+      if (! starting_dir)
+	error (1, errno, _("cannot get current directory"));
+    }
+  if ((*xstat) (".", &starting_stat_buf) != 0)
+    error (1, errno, _("cannot get current directory"));
 
   /* If no paths are given, default to ".".  */
   for (i = 1; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
@@ -281,12 +289,35 @@ main (int argc, char **argv)
   exit (exit_status);
 }
 
+/* Safely go back to the starting directory. */
+static void
+chdir_back (void)
+{
+  struct stat stat_buf;
+
+  if (starting_desc < 0)
+    {
+      if (chdir (starting_dir) != 0)
+	error (1, errno, "%s", starting_dir);
+      if ((*xstat) (".", &stat_buf) != 0)
+	error (1, errno, "%s", starting_dir);
+      if (stat_buf.st_dev != starting_stat_buf.st_dev ||
+	  stat_buf.st_ino != starting_stat_buf.st_ino)
+	error (1, 0, _("%s changed during execution of %s"), starting_dir, program_name);
+    }
+  else
+    {
+      if (fchdir (starting_desc) != 0)
+	error (1, errno, "%s", starting_dir);
+    }
+}
+
 /* Descend PATHNAME, which is a command-line argument.  */
 
 static void
 process_top_path (char *pathname)
 {
-  struct stat stat_buf;
+  struct stat stat_buf, cur_stat_buf;
 
   curdepth = 0;
   path_length = strlen (pathname);
@@ -303,14 +334,16 @@ process_top_path (char *pathname)
 	  exit_status = 1;
 	  return;
 	}
+
+      /* Check that we are where we should be. */
+      if ((*xstat) (".", &cur_stat_buf) != 0)
+	error (1, errno, "%s", pathname);
+      if (cur_stat_buf.st_dev != stat_buf.st_dev ||
+	  cur_stat_buf.st_ino != stat_buf.st_ino)
+	error (1, 0, _("%s changed during execution of %s"), pathname, program_name);
+
       process_path (pathname, ".", false, ".");
-#ifndef HAVE_FCHDIR
-      if (chdir (starting_dir) < 0)
-	error (1, errno, "%s", starting_dir);
-#else
-      if (fchdir (starting_desc))
-	error (1, errno, _("cannot return to starting directory"));
-#endif
+      chdir_back ();
     }
   else
     process_path (pathname, pathname, false, ".");
@@ -352,6 +385,8 @@ process_path (char *pathname, char *name, boolean leaf, char *parent)
   struct stat stat_buf;
   static dev_t root_dev;	/* Device ID of current argument pathname. */
   int i;
+  struct stat dir_buf;
+  int parent_desc;
 
   /* Assume it is a non-directory initially. */
   stat_buf.st_mode = 0;
@@ -445,6 +480,7 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 {
   char *name_space;		/* Names of files in PATHNAME. */
   int subdirs_left;		/* Number of unexamined subdirs in PATHNAME. */
+  struct stat stat_buf;
 
   subdirs_left = statp->st_nlink - 2; /* Account for name and ".". */
 
@@ -483,6 +519,13 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 	  return;
 	}
 
+      /* Check that we are where we should be. */
+      if ((*xstat) (".", &stat_buf) != 0)
+	error (1, errno, "%s", pathname);
+      if (stat_buf.st_dev != dir_ids[dir_curr].dev ||
+	  stat_buf.st_ino != dir_ids[dir_curr].ino)
+	error (1, 0, _("%s changed during execution of %s"), starting_dir, program_name);
+
       for (namep = name_space; *namep; namep += file_len - pathname_len + 1)
 	{
 	  /* Append this directory entry's name to the path being searched. */
@@ -521,22 +564,33 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 
       if (strcmp (name, "."))
 	{
+	  /* We could go back and do the next command-line arg
+	     instead, maybe using longjmp.  */
+	  char const *dir;
+
 	  if (!dereference)
-	    {
-	      if (chdir ("..") < 0)
-		/* We could go back and do the next command-line arg instead,
-		   maybe using longjmp.  */
-		error (1, errno, "%s", parent);
-	    }
+	    dir = "..";
 	  else
 	    {
-#ifndef HAVE_FCHDIR
-	      if (chdir (starting_dir) || chdir (parent))
-		error (1, errno, "%s", parent);
-#else
-	      if (fchdir (starting_desc) || chdir (parent))
-		error (1, errno, "%s", parent);
-#endif
+	      chdir_back ();
+	      dir = parent;
+	    }
+
+	  if (chdir (dir) != 0)
+	    error (1, errno, "%s", parent);
+
+	  /* Check that we are where we should be. */
+	  if ((*xstat) (".", &stat_buf) != 0)
+	    error (1, errno, "%s", pathname);
+	  if (stat_buf.st_dev !=
+	      (dir_curr > 0 ? dir_ids[dir_curr-1].dev : starting_stat_buf.st_dev) ||
+	      stat_buf.st_ino !=
+	      (dir_curr > 0 ? dir_ids[dir_curr-1].ino : starting_stat_buf.st_ino))
+	    {
+	      if (dereference)
+	        error (1, 0, _("%s changed during execution of %s"), parent, program_name);
+	      else
+	        error (1, 0, _("%s/.. changed during execution of %s"), starting_dir, program_name);
 	    }
 	}
 
@@ -560,3 +614,19 @@ no_side_effects (struct predicate *pred)
     }
   return (true);
 }
+
+/* Return true if there are no predicates with no_default_print in
+   predicate list PRED, false if there are any.
+   Returns true if default print should be performed */
+
+static boolean
+default_prints (struct predicate *pred)
+{
+  while (pred != NULL)
+    {
+      if (pred->no_default_print)
+	return (false);
+      pred = pred->pred_next;
+    }
+  return (true);
+}
