@@ -437,6 +437,7 @@ parse_fls (char **argv, int *arg_ptr)
   our_pred = insert_primary (pred_fls);
   our_pred->args.stream = open_output_file (argv[*arg_ptr]);
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   (*arg_ptr)++;
   return (true);
 }
@@ -478,6 +479,7 @@ parse_fprint (char **argv, int *arg_ptr)
   our_pred = insert_primary (pred_fprint);
   our_pred->args.stream = open_output_file (argv[*arg_ptr]);
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   our_pred->need_stat = false;
   (*arg_ptr)++;
   return (true);
@@ -493,6 +495,7 @@ parse_fprint0 (char **argv, int *arg_ptr)
   our_pred = insert_primary (pred_fprint0);
   our_pred->args.stream = open_output_file (argv[*arg_ptr]);
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   our_pred->need_stat = false;
   (*arg_ptr)++;
   return (true);
@@ -549,27 +552,28 @@ parse_help (char **argv, int *arg_ptr)
 {
   printf (_("\
 Usage: %s [path...] [expression]\n"), program_name);
-  printf (_("\
+  puts (_("\
 default path is the current directory; default expression is -print\n\
 expression may consist of:\n\
 operators (decreasing precedence; -and is implicit where no others are given):\n\
       ( EXPR ) ! EXPR -not EXPR EXPR1 -a EXPR2 EXPR1 -and EXPR2\n"));
-  printf (_("\
+  puts (_("\
       EXPR1 -o EXPR2 EXPR1 -or EXPR2 EXPR1 , EXPR2\n\
 options (always true): -daystart -depth -follow --help\n\
       -maxdepth LEVELS -mindepth LEVELS -mount -noleaf --version -xdev\n\
 tests (N can be +N or -N or N): -amin N -anewer FILE -atime N -cmin N\n"));
-  printf (_("\
+  puts (_("\
       -cnewer FILE -ctime N -empty -false -fstype TYPE -gid N -group NAME\n\
       -ilname PATTERN -iname PATTERN -inum N -ipath PATTERN -iregex PATTERN\n\
       -links N -lname PATTERN -mmin N -mtime N -name PATTERN -newer FILE\n"));
-  printf (_("\
+  puts (_("\
       -nouser -nogroup -path PATTERN -perm [+-]MODE -regex PATTERN\n\
       -size N[bckw] -true -type [bcdpfls] -uid N -used N -user NAME\n\
       -xtype [bcdpfls]\n"));
-  printf (_("\
+  puts (_("\
 actions: -exec COMMAND ; -fprint FILE -fprint0 FILE -fprintf FILE FORMAT\n\
       -ok COMMAND ; -print -print0 -printf FORMAT -prune -ls\n"));
+  puts (_("\nReport bugs to <bug-findutils@gnu.org>."));
   exit (0);
 }
 
@@ -652,6 +656,7 @@ parse_ls (char **argv, int *arg_ptr)
 
   our_pred = insert_primary (pred_ls);
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   return (true);
 }
 
@@ -962,6 +967,7 @@ parse_print (char **argv, int *arg_ptr)
      from doing undesired multiple printing when the user has
      already specified -print. */
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   our_pred->need_stat = false;
   return (true);
 }
@@ -976,6 +982,7 @@ parse_print0 (char **argv, int *arg_ptr)
      from doing undesired multiple printing when the user has
      already specified -print0. */
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   our_pred->need_stat = false;
   return (true);
 }
@@ -1236,6 +1243,11 @@ insert_type (char **argv, int *arg_ptr, boolean (*which_pred) (/* ??? */))
     case 's':			/* socket */
       type_cell = S_IFSOCK;
       break;
+#endif
+#ifdef S_IFDOOR
+    case 'D':			/* Solaris door */
+      type_cell = S_IFDOOR;
+      break;
 #endif
     default:			/* None of the above ... nuke 'em. */
       return (false);
@@ -1264,6 +1276,7 @@ insert_fprintf (FILE *fp, boolean (*func) (/* ??? */), char **argv, int *arg_ptr
   fprintf_stat_needed = false;	/* Might be overridden later. */
   our_pred = insert_primary (func);
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   our_pred->args.printf_vec.stream = fp;
   segmentp = &our_pred->args.printf_vec.segment;
   *segmentp = NULL;
@@ -1445,6 +1458,7 @@ make_segment (struct segment **segment, char *format, int len, int kind)
   return (&(*segment)->next);
 }
 
+/* handles both exec and ok predicate */
 static boolean
 insert_exec_ok (boolean (*func) (/* ??? */), char **argv, int *arg_ptr)
 {
@@ -1475,6 +1489,7 @@ insert_exec_ok (boolean (*func) (/* ??? */), char **argv, int *arg_ptr)
 
   our_pred = insert_primary (func);
   our_pred->side_effects = true;
+  our_pred->no_default_print = true;
   execp = &our_pred->args.exec_vec;
   execp->paths =
     (struct path_arg *) xmalloc (sizeof (struct path_arg) * (num_paths + 1));
