@@ -1133,6 +1133,9 @@ pred_type (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 #endif
 #ifdef S_IFSOCK
       || (S_ISSOCK (mode) && type == S_IFSOCK)
+#endif
+#ifdef S_IFDOOR
+      || (S_ISDOOR (mode) && type == S_IFDOOR)
 #endif
       )
 #else /* S_IFMT */
@@ -1263,19 +1266,13 @@ launch (struct predicate *pred_ptr)
   if (child_pid == 0)
     {
       /* We be the child. */
-#ifndef HAVE_FCHDIR
-      if (chdir (starting_dir) < 0)
+      if (starting_desc < 0
+	  ? chdir (starting_dir) != 0
+	  : fchdir (starting_desc) != 0)
 	{
 	  error (0, errno, "%s", starting_dir);
 	  _exit (1);
 	}
-#else
-      if (fchdir (starting_desc) < 0)
-	{
-	  error (0, errno, _("cannot return to starting directory"));
-	  _exit (1);
-	}
-#endif
       execvp (execp->vec[0], execp->vec);
       error (0, errno, "%s", execp->vec[0]);
       _exit (1);
