@@ -34,6 +34,9 @@
 #ifdef S_ISSOCK
 #define S_IFSOCK 64
 #endif
+#ifdef S_ISDOOR
+#define S_IFDOOR 128
+#endif
 #endif /* !S_IFMT */
 
 #ifdef STAT_MACROS_BROKEN
@@ -44,6 +47,7 @@
 #undef S_ISFIFO
 #undef S_ISLNK
 #undef S_ISSOCK
+#undef S_ISDOOR
 #undef S_ISMPB
 #undef S_ISMPC
 #undef S_ISNWK
@@ -72,6 +76,9 @@
 #if !defined(S_ISSOCK) && defined(S_IFSOCK)
 #define	S_ISSOCK(m) (((m) & S_IFMT) == S_IFSOCK)
 #endif
+#if !defined(S_ISDOOR) && defined(S_IFDOOR)
+#define	S_ISDOOR(m) (((m) & S_IFMT) == S_IFDOOR)
+#endif
 #if !defined(S_ISMPB) && defined(S_IFMPB) /* V7 */
 #define S_ISMPB(m) (((m) & S_IFMT) == S_IFMPB)
 #define S_ISMPC(m) (((m) & S_IFMT) == S_IFMPC)
