@@ -76,8 +76,9 @@
 #endif
 
 #include "locatedb.h"
+#include <getline.h>
 
-char *xmalloc ();
+char *xmalloc PARAMS((size_t));
 
 /* The name this program was run with.  */
 char *program_name;
@@ -85,9 +86,7 @@ char *program_name;
 /* Write out a 16-bit int, high byte first (network byte order).  */
 
 static void
-put_short (c, fp)
-     int c;
-     FILE *fp;
+put_short (int c, FILE *fp)
 {
   putc (c >> 8, fp);
   putc (c, fp); 
@@ -96,8 +95,7 @@ put_short (c, fp)
 /* Return the length of the longest common prefix of strings S1 and S2. */
 
 static int
-prefix_length (s1, s2)
-     char *s1, *s2;
+prefix_length (char *s1, char *s2)
 {
   register char *start;
 
@@ -106,10 +104,8 @@ prefix_length (s1, s2)
   return s1 - start;
 }
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   char *path;			/* The current input entry.  */
   char *oldpath;		/* The previous input entry.  */
@@ -119,7 +115,7 @@ main (argc, argv)
 
   program_name = argv[0];
 
-  pathsize = oldpathsize = 1026; /* Increased as necessary by getstr.  */
+  pathsize = oldpathsize = 1026; /* Increased as necessary by getline.  */
   path = xmalloc (pathsize);
   oldpath = xmalloc (oldpathsize);
 
@@ -132,7 +128,7 @@ main (argc, argv)
 
   /* FIXME temporary: change the \n to \0 when we figure out how to sort
      null-terminated strings.  */
-  while ((line_len = getstr (&path, &pathsize, stdin, '\n', 0)) > 0)
+  while ((line_len = getline (&path, &pathsize, stdin)) > 0)
     {
       path[line_len - 1] = '\0'; /* FIXME temporary: nuke the newline.  */
 
